import os
from typing import Tuple

from netCDF4 import Dataset
import numpy as np
import re
from oscars_mgmnt_clms.file_object import ObjectInfo
from oscars_mgmnt_clms.parser.filename import (
    FilenameParser,
    get_geometry,
    decode_platforms,
    parse_date,
    format_date,
    fix_bbox,
)


class CLMSGlobalParser_netCDF(FilenameParser):
    """Parser used to build GeoJSON metadata from CLMS Global Land products in netCDF format"""

    @property
    def filetype(self) -> str:
        return "netcdf"

    def __init__(self):
        super().__init__(template="CLMS_GLOBAL.template")

    def specific_treatment(self, product_file_info: ObjectInfo, values: dict):
        self.fix_products(product_file_info)

        # examples of supported filenames:
        #    c_gls_NDVI300_202204110000_GLOBE_OLCI_V2.0.1.nc
        #    c_gls_BA300-NRT_201901010000_GLOBE_S3_V3.1.1.nc
        fn = os.path.abspath(product_file_info.get_product_path())
        file_name = os.path.basename(fn).rpartition(".")[0]
        file_name_parts = file_name.split("_")

        ds = Dataset(fn)

        values["id"] = file_name
        values["title"] = ds.title

        (product_type, separator, product_group) = file_name_parts[2].partition("-")
        product_group = product_group.replace("STATIC-", "")
        if product_group:
            values["productgroupid"] = product_group

        values["producttype"] = product_type
        version = file_name_parts[-1]
        values["productVersion"] = version

        processingCenter = "VITO"
        if "institution" in ds.ncattrs():
            processingCenter = ds.institution.split(",")[0].upper()
        elif "processing_centre" in ds.ncattrs():
            processingCenter = ds.processing_centre.split(",")[0].upper()
        values["processingCenter"] = (
            processingCenter.replace(" NV", "").replace(" GMBH", "").replace(" ", "_")
        )

        # Coordinate rounding to 10 decimals
        # Allow one more digit for LST to avoid ES map parser error
        rounding = 11 if "land_surface_temperature" in fn else 10
        bbox = self.get_bbox(ds, rounding)
        (
            values["westbbox"],
            values["southbbox"],
            values["eastbbox"],
            values["northbbox"],
        ) = bbox
        values["geometry"] = get_geometry(bbox)

        # LWQ refers to GLOBAL, rename to GLOBE for consistency with other products
        values["tileid"] = file_name_parts[-3].replace("GLOBAL", "GLOBE")

        if "time_coverage_start" in ds.ncattrs():
            values["beginposition"] = format_date(parse_date(ds.time_coverage_start))
            values["endposition"] = format_date(parse_date(ds.time_coverage_end))
        else:
            values["beginposition"] = values["nominaldatetime"]
            values["endposition"] = values["nominaldatetime"]

        # collection_dir can have 12.5km, 5km, 1km, 300m, 100m or no resolution info
        collection_dir = re.match(
            f"clms_global_([\\w.-]+)_{self.filetype}", values["parentidentifier"]
        ).group(1)
        resolution_regex = re.compile("([\\d.]+)(m|km)")
        regex_result = resolution_regex.match(collection_dir.split("_")[1])
        if regex_result and product_group not in ["DGG", "TS"]:
            factor = 1000 if regex_result.group(2)[0] == "k" else 1
            values["resolution"] = int(float(regex_result.group(1)) * factor)
        else:
            values["resolution"] = None

        values["platforms"] = decode_platforms(ds.platform)

        # TODO extend for SWI DGG product
        values["conformsto"] = "https://www.opengis.net/def/crs/EPSG/0/4326"

        ds.close()

        return values["id"]

    def get_bbox(self, ds: Dataset, rounding: int) -> Tuple[float, float, float, float]:
        # netCDF stores center of pixel coordinates, OSCARS stores pixel top left coordinates
        # so we need to adjust with half a pixel
        westcenter = np.min(ds.variables["lon"])
        southcenter = np.min(ds.variables["lat"])
        eastcenter = np.max(ds.variables["lon"])
        northcenter = np.max(ds.variables["lat"])

        # SWI Time series: DGG grid is irregular in lat/lon degrees, no half pixel adjustment
        rasterxsize = ds.dimensions["lon"].size if "lon" in ds.dimensions else None
        rasterysize = ds.dimensions["lat"].size if "lat" in ds.dimensions else None

        return fix_bbox(
            (westcenter, southcenter, eastcenter, northcenter),
            rounding,
            rasterxsize,
            rasterysize,
        )

    @property
    def mimetype(self) -> str:
        return "application/netcdf"
